﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using HANDLE = System.IntPtr;

namespace vcsharp_interrupt_demo_v1._0
{

    public partial class Form1 : Form
    {
        [DllImport("Coredll.dll", CharSet = CharSet.Auto)]
        public static extern HANDLE CreateEvent(IntPtr lpEventAttributes, bool bManualReset, bool bInitialState, string lpName);

        [DllImport("Coredll.dll", CharSet = CharSet.Auto)]
        public static extern int WaitForSingleObject(HANDLE hHandle, int dwMilliseconds);

        [DllImport("Coredll.dll", CharSet = CharSet.Auto)]
        public static extern bool CloseHandle(HANDLE hObject);

        private const int   GPIO_EDGE_RISING    = 1;                                ///< Constant for rising edge
        private const int   GPIO_EDGE_FALLING   = 2;                                ///< Constant for falling edge
        private const int   GPIO_EDGE_BOTH      = 3;                                ///< Constant for both rising and falling
        private const int   WAIT_OBJECT_0       = 0;
        private const int   WAIT_TIMEOUT        = 0x102;

        private IntPtr      hGpio           = IntPtr.Zero;                          ///< Handle to gpio library
        private IntPtr      hEvent          = HANDLE.Zero;                          ///< Handle to Event for generated interrupt
        private IntPtr      hIntr           = IntPtr.Zero;                          ///< Handle to interrupt library
        private gpio.uIo    interruptPin    = new gpio.uIo();                       ///< gpio.uIo variable for holding interrupt pin 
        private gpio.uIo    outputPin       = new gpio.uIo();                       ///< gpio.uIo variable for holding output logic pin
        private UInt32      irqNum          = 0;                                    ///< variable for interrupt number
        private UInt32      sysIrq          = 0;                                    ///< variable for system interrupt number
        private int         interruptEdge   = GPIO_EDGE_RISING;                     ///< Triggering edge Falling, Rising, both.
        private int         interruptCount  = 0;                                    ///< Counting variable for number of interrupt
        private bool        initialized     = false;                                ///< Init flag for GUI code
        private bool        edgeChanged     = false;                                ///< Triggering edge change flag
        private bool        outputPinLogic = false;                                 ///< Output change flag for output pin    
        private bool        systemInterruptGenerated = false;                       ///< Interrupt generated flag

        public Form1()
        {
            InitializeComponent();
            edgeSelectComboBox.SelectedIndex = 0;
            /// Configures and initializes the interrupt.
            interruptPin.type = (ushort)gpio.tIoType.ioColibriPin;                  ///< SODIMM pin 101 as GPIO input to capture the interrupt.
            interruptPin.number = 101;
            InterruptConfig();                                                      ///< Interrupt init function call
            outputPin.type = (ushort)gpio.tIoType.ioColibriPin;                     ///< SODIMM pin 135 as GPIO output to blink the LED.
            outputPin.number = 135;                             
            gpio.Gpio_ConfigureAsGpio(hGpio, outputPin);                            ///< setting SODIMM pin 135 as gpio
            gpio.Gpio_SetDir(hGpio, outputPin, gpio.tIoDirection.ioOutput);         ///< setting SODIMM pin 135 direction as output
        }
        private bool InterruptConfig()
        {
            bool retVal = false;
            hGpio = gpio.Gpio_Init(null);                                           ///< Init gpio library handle
            gpio.Gpio_Open(hGpio);                                                  ///< Open gpio library                
            gpio.Gpio_ConfigureAsGpio(hGpio, interruptPin);                         ///< Configure io as GPIO
            gpio.Gpio_SetDir(hGpio, interruptPin, gpio.tIoDirection.ioInput);       ///< Set GPIO Configured io as input
            interruptPin = gpio.Gpio_NormalizeIo(hGpio, interruptPin);              ///< Convert IO (ioColibriPin, ioApalis or ioGpio) to ioGpio
            hIntr = Int.Int_Init();                                                 ///< Initialize interrupt Library handle
            if (hIntr == null)
            {
                MessageBox.Show("ERROR:: interrupt init function");
                return false;
            }
            SetEdgeTrigger(GPIO_EDGE_RISING);                                       ///< Set interrupt edge trigger funtion
            unsafe
            {
                gpio.Gpio_GetConfigInt(hGpio, interruptPin, "irq", ref irqNum);     ///< Get Irq number from GPIO number
            }
            hEvent = CreateEvent(IntPtr.Zero, false, false, null);                  ///< Create an Eventto handle gpio interrupt
            if (hEvent == HANDLE.Zero)
            {
                MessageBox.Show("Failed to generate event.", "Error");
                return false;
            }
            sysIrq = Int.Int_RequestSysInterrupt(hIntr, irqNum);                    ///< Get the system interrupt for the corresponding Irq number
            if (sysIrq == 0)
            {
                MessageBox.Show("Failed to get system interrupt number.", "Error");
                return false;
            }
            systemInterruptGenerated = true;
            retVal = Int.Int_InterruptInitialize(hIntr, sysIrq, hEvent, IntPtr.Zero, 0);///< Initialize interrupt
            if (retVal == false)
            {
                MessageBox.Show("Failed to initialize the interrupt.", "Error");
                return false;
            }
            initialized = true;
            edgeSelectComboBox.Enabled = true;
            startButton.Enabled = true;
            return true;
        }

        private bool SetEdgeTrigger(int edgetriggered)
        {
            switch (edgetriggered)
            {
                case GPIO_EDGE_RISING:
                    gpio.Gpio_SetConfigString(hGpio, interruptPin, null, "irqtrig=rising", TdxCommon.ParamStorageType.StoreVolatile); ///< Set Triggering edge to rising 
                    break;
                case GPIO_EDGE_FALLING:
                    gpio.Gpio_SetConfigString(hGpio, interruptPin, null, "irqtrig=falling", TdxCommon.ParamStorageType.StoreVolatile); ///< Set Triggering edge to falling
                    break;
                case GPIO_EDGE_BOTH:
                    gpio.Gpio_SetConfigString(hGpio, interruptPin, null, "irqtrig=both", TdxCommon.ParamStorageType.StoreVolatile);    ///< Set Triggering edge to both
                    break;
            }
            return true;
        }
        /// <summary>
        /// Restart interrupt routine
        /// </summary>
        /// <returns></returns>
        private bool RestartInterrupt()
        {
            bool retVal = false;
            if (initialized == true)
            {
                initialized = false;
                Int.Int_InterruptDisable(hIntr, sysIrq);                                       ///< Disables system interrupt.
                SetEdgeTrigger(interruptEdge);                                                 ///< Set Triggering edge function 
                retVal = Int.Int_InterruptInitialize(hIntr, sysIrq, hEvent, IntPtr.Zero, 0);   ///< Initialize interrupt
                initialized = true;
            }
            return retVal;
        }
        /// <summary>
        /// selecting triggering edge from combobox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void edgeSelectComboBox_SelectedIndexChanged_1(object sender, EventArgs e)
        {
            interruptEdge = edgeSelectComboBox.SelectedIndex + 1;
            edgeChanged = true;
        }
        /// <summary>
        /// Start to capture interrupt when press on start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            bool retVal = false;
            if (initialized == true)                    ///< Starts only if initialization is done.
            {
                if (startButton.Text == "Start")
                {
                    edgeSelectComboBox.Enabled = false;
                    if (edgeChanged == true)
                    {
                        edgeChanged = false;
                        retVal = RestartInterrupt();    ///< Restarts interrupt to change the edge.
                        if (retVal == false)
                        {
                            textBox1.Text = "Failed.";
                            startButton.Enabled = false;
                            return;
                        }
                    }
                    startButton.Text = "Stop";
                    scanTimer.Enabled = true;
                }
                else
                {
                    scanTimer.Enabled = false;
                    System.Threading.Thread.Sleep(10);
                    edgeSelectComboBox.Enabled = true;
                    startButton.Text = "Start";
                }
            }
        }

        /// <summary>
        /// Clear counted interrupt and set to zero
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void clearButton_Click(object sender, EventArgs e)
        {
            interruptCount = 0;
            textBox1.Text = "0";
        }
        /// <summary>
        /// Timer to report interrupt genrated from gpio
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void scanTimer_Tick(object sender, EventArgs e)
        {
            int retVal = 0;
            retVal = WaitForSingleObject(hEvent, 10);   ///< Waits for event associated with interrupt.
            if (retVal == WAIT_OBJECT_0)
            {
                interruptCount++;
                textBox1.Text = interruptCount.ToString();
                if (outputPinLogic == false)
                {
                    gpio.Gpio_SetLevel(hGpio, outputPin, gpio.tIoLevel.ioHigh); ///< Set output pin (SODIMM 135) to high
                    outputPinLogic = true;
                }
                else
                {
                    gpio.Gpio_SetLevel(hGpio, outputPin, gpio.tIoLevel.ioLow);  ///< Set output pin (SODIMM 135) to low
                    outputPinLogic = false;
                }
                Int.Int_InterruptDone(hIntr, sysIrq);                           ///< Clears the interrupt generated.
            }
            else if (retVal != WAIT_TIMEOUT)
            {
                textBox1.Text = "Failed.";
                startButton_Click(sender, e);
            }
        }
        /// <summary>
        /// De-init all handle and library on Form Closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Closed(object sender, EventArgs e)
        {
            scanTimer.Enabled = false;
            System.Threading.Thread.Sleep(10);
            ///< Disables system interrupt.
            ///< It is very important to deinitalise the interrupt, otherwise it will not work the next time you start the program
            Int.Int_InterruptDisable(hIntr, sysIrq);
            if (hEvent != HANDLE.Zero)
            {
                CloseHandle(hEvent);
            }
            if (systemInterruptGenerated == true)
            {
                Int.Int_ReleaseSysIntr(hIntr, sysIrq);                              ///< Releases all the resources related to system interrupt.
                Int.Int_Deinit(hIntr);                                              ///< De-initializes Interrupt handle
            }
            gpio.Gpio_Close(hGpio);                                                 ///< Close the GPIO library.
            gpio.Gpio_Deinit(hGpio);                                                ///< De-initializes GPIO handle
        }
    }
}